/* Copyright (C) 2000-2013 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#ifndef _UDM_COMMON_H
#define _UDM_COMMON_H

#include "udm_config.h"

#include <stdio.h> /* for FILE etc. */

#include <sys/types.h>
#include "udm_regex.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_WINSOCK_H
#include <winsock.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif
#ifdef HAVE_ARPA_NAMESER_H
#include <arpa/nameser.h>
#endif
#ifdef HAVE_RESOLV_H
#include <resolv.h>
#endif
#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif
#ifdef MECAB
#include <mecab.h>
#endif

#ifdef HAVE_DEBUG
#include <assert.h>
#endif

/************************ Basic data types ****************/
typedef enum 
{
  UDM_FALSE= 0,
  UDM_TRUE= 1
} udm_bool_t;

typedef uint4 udm_pos_t;
typedef unsigned char udm_secno_t;
typedef unsigned char udm_wordnum_t;

/**********************************************************/

#include "udm_unicode.h"
#include "udm_uniconv.h"
#include "udm_unidata.h"
#include "udm_utils.h"
#include "udm_hash.h"

extern int new_version;

/* Some constants */
#define UDM_LANGPERDOC				16		/* FIXME */
#define UDM_USER_AGENT				"MnoGoSearch/" VERSION
#define UDM_MAXWORDPERQUERY			64

/* Some sizes and others definitions */
#define UDM_MAXDOCSIZE				2*1024*1024	/**< 2 MB  */
#define UDM_DEFAULT_REINDEX_TIME		7*24*60*60	/**< 1week */
#define UDM_MAXWORDSIZE				32
#define UDM_MAXDISCWORDSIZE			64
#define UDM_DEFAULT_MAX_HOPS			256
#define UDM_READ_TIMEOUT			30
#define UDM_DOC_TIMEOUT				90
#define UDM_MAXNETERRORS			16
#define UDM_DEFAULT_NET_ERROR_DELAY_TIME	86400
#define UDM_DEFAULT_BAD_SINCE_TIME              15*24*60*60     /**< 15 days */
#define UDM_FINDURL_CACHE_SIZE                  128
#define UDM_SERVERID_CACHE_SIZE                 128


/* storage types */
#define UDM_DBMODE_SINGLE	0
#define UDM_DBMODE_MULTI	1
#define UDM_DBMODE_BLOB		6
#define UDM_DBMODE_RAWBLOB	7


/* database open modes */
#define UDM_OPEN_MODE_READ	0
#define UDM_OPEN_MODE_WRITE	1

/* search modes */
#define UDM_MODE_ALL             0
#define UDM_MODE_ANY             1
#define UDM_MODE_BOOL            2
#define UDM_MODE_PHRASE          3
#define UDM_MODE_ALL_MINUS       4
#define UDM_MODE_ALL_MINUS_HALF  5

/* word match type */
#define UDM_MATCH_FULL		0
#define UDM_MATCH_BEGIN		1
#define UDM_MATCH_SUBSTR	2
#define UDM_MATCH_END		3
#define UDM_MATCH_REGEX		4
#define UDM_MATCH_WILD		5
#define UDM_MATCH_SUBNET	6
#define UDM_MATCH_NUMERIC_LT	7
#define UDM_MATCH_NUMERIC_GT	8
#define UDM_MATCH_RANGE         9

/* Case sensitivity */
#define UDM_CASE_SENSITIVE      0
#define UDM_CASE_INSENSITIVE    1

/* Indexer return codes */
#define UDM_OK			0
#define UDM_ERROR		1
#define UDM_NOTARGET		2
#define UDM_TERMINATED		3

/* Flags for indexing */
#define UDM_FLAG_REINDEX           1
#define UDM_FLAG_SORT_EXPIRED      2
#define UDM_FLAG_SORT_HOPS         4
#define UDM_FLAG_ADD_SERV          8
#define UDM_FLAG_SPELL	          16
#define UDM_FLAG_LOAD_LANGMAP	    32
#define UDM_FLAG_DONTSORT_SEED    64
#define UDM_FLAG_ADD_SERVURL	   128
#define UDM_FLAG_DONT_ADD_TO_DB  256


/* URLFile actions */
#define UDM_URL_FILE_REINDEX	1
#define UDM_URL_FILE_CLEAR	2
#define UDM_URL_FILE_INSERT	3
#define UDM_URL_FILE_PARSE	4

/* Ispell mode binary flags */
#define UDM_ISPELL_MODE_DB	1
#define UDM_ISPELL_USE_PREFIXES	2
#define UDM_ISPELL_MODE_SERVER  4


/* Known content types */
typedef enum 
{
  UDM_CONTENT_TYPE_UNKNOWN= 0,
  UDM_CONTENT_TYPE_TEXT_PLAIN= 1,
  UDM_CONTENT_TYPE_TEXT_HTML= 2,
  UDM_CONTENT_TYPE_TEXT_XML= 3,
  UDM_CONTENT_TYPE_MESSAGE_RFC822= 4,
  UDM_CONTENT_TYPE_AUDIO_MPEG= 5,
  UDM_CONTENT_TYPE_HTDB= 6,
  UDM_CONTENT_TYPE_DOCX= 7,
  UDM_CONTENT_TYPE_TEXT_RTF= 8
} udm_content_type_t;


/* Action type: HTTP methods */
typedef enum
{
  UDM_METHOD_GET=          0,
  UDM_METHOD_UNKNOWN=      1,
  UDM_METHOD_DISALLOW=     2,
  UDM_METHOD_HEAD=         3,
  UDM_METHOD_HREFONLY=     4,
  UDM_METHOD_CHECKMP3=     5,
  UDM_METHOD_CHECKMP3ONLY= 6,
  UDM_METHOD_VISITLATER=   7,
  UDM_METHOD_INDEX=        8,
  UDM_METHOD_NOINDEX=      9,
  UDM_METHOD_IMPORTONLY=   10
} udm_method_t;

#define UDM_METHOD_DEFAULT      UDM_METHOD_GET

typedef enum
{
  UDM_LINK_SOURCE_UNKNOWN=         0,
  UDM_LINK_SOURCE_CMDLINE=         1,
  UDM_LINK_SOURCE_CONF=            2,
  UDM_LINK_SOURCE_A_HREF=          3,
  UDM_LINK_SOURCE_FRAME_SRC=       4,
  UDM_LINK_SOURCE_IMG_SRC=         5,
  UDM_LINK_SOURCE_REDIRECT=        6,
  UDM_LINK_SOURCE_META_REFRESH=    7,
  UDM_LINK_SOURCE_LINK_HREF=       8,
  UDM_LINK_SOURCE_AREA_HREF=       9,
  UDM_LINK_SOURCE_IFRAME_SRC=     10,
  UDM_LINK_SOURCE_SCRIPT_SRC=     11,
  UDM_LINK_SOURCE_HTDB=           12,
  UDM_LINK_SOURCE_URLFILE=        13,
  UDM_LINK_SOURCE_ROBOTS_SITEMAP= 14,
  UDM_LINK_SOURCE_XML=            15
} udm_link_source_t;
#define UDM_LINK_SOURCES_YES 0x7FFFFFFF


/* CollectLinks flags*/
typedef enum
{
  UDM_COLLECT_LINKS_NONE=      0x0000,  /* do not collect links           */
  UDM_COLLECT_LINKS_INNER=     0x0001,  /* links covered by .conf file    */
  UDM_COLLECT_LINKS_OUTER=     0x0002,  /* links going outside .conf file */
  UDM_COLLECT_LINKS_SITE=      0x0004,  /* links going to the same site   */
  UDM_COLLECT_LINKS_PAGE=      0x0008,  /* links going to the same page   */
  UDM_COLLECT_LINKS_BADSCHEME= 0x0010,  /* Schema: mailto, javascript     */
  UDM_COLLECT_LINKS_BAD=       0x0020,  /* Bad syntax                     */
  UDM_COLLECT_LINKS_HOPS=      0x0040,  /* Too many hops                  */
  UDM_COLLECT_LINKS_FILTER=    0x0080,  /* Too many hops                  */
  UDM_COLLECT_LINKS_PERSITE=   0x0100,  /* Per site limite reached        */
  UDM_COLLECT_LINKS_ALL_DST=   0xFFFF
} udm_link_destination_t;


/* Robots flags */
typedef enum
{
  UDM_ROBOTS_NONE=   0x0000, /* Don't respect robots limitation  */
  UDM_ROBOTS_TXT=    0x0001, /* Respect robots.txt instructions  */
  UDM_ROBOTS_HEADER= 0x0002, /* Restext "X-Robots-Tag" HTTP header        */
  UDM_ROBOTS_META=   0x0004, /* Respect <meta robots="xxx">  insturctions */
  UDM_ROBOTS_REL=    0x0008, /* Respect <link rel="nofollow> insturctions */
  UDM_ROBOTS_ALL=    0xFFFF  /* Respect all robots instructions  */
} udm_use_robots_t;


typedef enum
{
  UDM_COLLECT_LINKS_ABSOLUTE=  0x000000, /* Store in absolute format   */
  UDM_COLLECT_LINKS_ASIS=      0x010000, /* Preserve the original form */
  UDM_COLLECT_LINKS_FORMAT=    0xFF0000
} udm_link_format_t;


/* Content encoding types */
typedef enum udm_content_encoding_enum
{
  UDM_CONTENT_ENCODING_IDENTITY= 0,
  UDM_CONTENT_ENCODING_DEFLATE=  1,
  UDM_CONTENT_ENCODING_GZIP=     3,
  UDM_CONTENT_ENCODING_ZLIB_COMPRESS= 4,
  UDM_CONTENT_ENCODING_UNKNOWN=  5
} udm_content_encoding_t;


/* Words origins */
#define UDM_WORD_ORIGIN_QUERY           1
#define UDM_WORD_ORIGIN_SPELL           2
#define UDM_WORD_ORIGIN_SYNONYM         3
#define UDM_WORD_ORIGIN_SYNONYM_FINAL   4
#define UDM_WORD_ORIGIN_STOP            5
#define UDM_WORD_ORIGIN_SUGGEST         6
#define UDM_WORD_ORIGIN_COLLATION       7

/* URL data flags */
#define UDM_URLDATA_URL	        1
#define UDM_URLDATA_SITE        2
#define UDM_URLDATA_POP         4
#define UDM_URLDATA_LM          8 
#define UDM_URLDATA_SU         16
#define UDM_URLDATA_CACHE      32
#define UDM_URLDATA_SITE_RANK  64

/* Locking mutex numbers */
#define UDM_LOCK_DUMMY          0
#define UDM_LOCK_CONF		1
#define UDM_LOCK_WIN		2
#define UDM_LOCK_THREAD         3
#define UDM_LOCK_SQL            4
#define UDM_LOCK_SEGMENTER      5
#define UDM_LOCK_DB             6
#define UDM_LOCK_LOG		7
#define UDM_LOCK_ROBOT_FIRST	8
#define UDM_LOCK_ROBOT_LAST	128
#define UDM_LOCK_ROBOT_COUNT    (UDM_LOCK_ROBOT_LAST - UDM_LOCK_ROBOT_FIRST +1)
#define UDM_LOCK_HOST_FIRST     129
#define UDM_LOCK_HOST_LAST      255
#define UDM_LOCK_HOST_COUNT     (UDM_LOCK_HOST_LAST - UDM_LOCK_HOST_FIRST + 1)
#define UDM_LOCK_MAX            (UDM_LOCK_HOST_LAST + 1)

/************************ Statistics **********************/
typedef struct stat_struct {
	int	status;
	int	expired;
	int	total;
} UDM_STAT;

typedef struct stat_list_struct{
	time_t		time;
	size_t		nstats;
	UDM_STAT	*Stat;
} UDM_STATLIST;
/************************ VARLISTs ************************/

/* Various variable flags */
typedef enum
{
  UDM_VARFLAG_NOCLONE=    0x01,  /* Ignore for clone detection        */ 
  UDM_VARFLAG_USERDEF=    0x02,  /* User defined section              */
  UDM_VARFLAG_KEEPVAL=    0x04,  /* If value doesn't need to be freed */
  UDM_VARFLAG_HTMLSOURCE= 0x08,  /* If apply HTML parser, for HTDB    */
  UDM_VARFLAG_WIKI=       0x10,  /* If to remove text between [ and ] */
  UDM_VARFLAG_HL=         0x20,  /* If variable has highlight markers */
  UDM_VARFLAG_NOINDEX=    0x40,  /* If section should be in bdicti but not in bdict */
  UDM_VARFLAG_DECIMAL=    0x80,  /* Whether to detect decimal numbers */
  UDM_VARFLAG_RAW=       0x100   /* If sections is Raw (no cs conversion */
} udm_var_flag_t;


/* Variable types */
#define UDM_VAR_INT       1
#define UDM_VAR_STR       2
#define UDM_VAR_DOUBLE    3
#define UDM_VAR_ENV       16
#define UDM_VAR_AGENT     17
#define UDM_VAR_RESULT    18
#define UDM_VAR_DOCUMENT  19
#define UDM_VAR_SQLRESULT 20

#define UDM_VARSRC_QSTRING 1
#define UDM_VARSRC_ENV     2

struct udm_var_handler_st;

typedef struct udm_var_st
{
  struct udm_var_handler_st *handler; /* Variable handler */
  size_t         maxlen;    /**< Max length      */
  size_t         curlen;    /**< Cur length      */
  char           *val;      /**< Field Value     */
  char           *name;     /**< Field Name      */
  udm_secno_t    section;   /**< Number 0..255   */
  udm_var_flag_t flags;
} UDM_VAR;


typedef struct udm_varlist_st
{
  int      freeme;
  size_t   nvars; /* Number of registered variables */
  size_t   mvars; /* Number of allocated variables  */
  size_t   svars; /* Number of sorted variables     */
  UDM_VAR  *Var;
} UDM_VARLIST;


typedef enum
{
  UDM_TEXTLIST_FLAG_SKIP_ADD_SECTION= 0x01,
  UDM_TEXTLIST_FLAG_RFC1522=          0x02, /* Message header (Subj, From)*/
  UDM_TEXTLIST_FLAG_MESSAGE_RFC822=   0x04, /* Used by cached copy for messages */
  UDM_TEXTLIST_FLAG_HTML=             0x08  /* HTML format with entities */
} udm_textlist_flag_t;


typedef struct
{
  UDM_CONST_STR text;
  UDM_CONST_STR href;
  UDM_CONST_STR section_name;
  udm_secno_t section;
  udm_textlist_flag_t flags;
} UDM_CONST_TEXTITEM;


typedef struct
{
  char  *str;
  char  *href;
  char  *section_name;
  udm_secno_t section;
  udm_textlist_flag_t flags;
} UDM_TEXTITEM;


typedef struct
{
  size_t        nitems;
  size_t        mitems;
  UDM_TEXTITEM	*Item;
} UDM_TEXTLIST;

/*****************************************************/

/** StopList unit */
typedef struct udm_stopword_struct
{
  char *word;
} UDM_STOPWORD;

#define UDM_STOPLIST_LANGLEN 32
#define UDM_STOPLIST_CSETLEN 32
#define UDM_STOPLIST_FILELEN 128

typedef struct
{
  size_t nstopwords;
  UDM_STOPWORD	*StopWord;
  char  lang[UDM_STOPLIST_LANGLEN];
  char  cset[UDM_STOPLIST_CSETLEN];
  char  fname[UDM_STOPLIST_FILELEN];
} UDM_STOPLIST;

typedef struct
{
  size_t nitems;
  UDM_STOPLIST *Item;
} UDM_STOPLISTLIST;

/*****************************************************/

/** Words parameters */
typedef struct {
	size_t		min_word_len;
	size_t		max_word_len;
} UDM_WORDPARAM;


/**************************/
typedef struct udm_coord_st
{
  udm_pos_t pos:24;   /* 3 */
  udm_secno_t secno;  /* 1 */
} UDM_COORD;

typedef struct udm_urlid_coord_st
{
  urlid_t url_id;     /* 4 */
  UDM_COORD coord;    /* 4 */
} UDM_URLID_COORD;

typedef struct
{
  UDM_URLID_COORD urlid_coord;
  udm_pos_t     seclen:24; /* 3 */
  udm_wordnum_t num;       /* 1 */
} UDM_URL_CRD;             /* 12 bytes total */


typedef struct
{
  size_t       acoords;
  size_t       ncoords;
  size_t       order;
  char	       *word;
  UDM_URL_CRD  *Coords;
} UDM_URLCRDLIST;


/***************************/
typedef struct udm_coord2_st
{
  udm_pos_t pos:24;
  udm_wordnum_t order;
} UDM_COORD2;


typedef struct udm_section_st
{
  UDM_COORD2 *Coord;                 /* 4/8 bytes */
  const unsigned char *PackedCoord;  /* 4/8 bytes */
  urlid_t url_id;                    /* 4   bytes */
  udm_pos_t ncoords;                 /* 4   bytes */
  udm_pos_t seclen;                  /* 4   bytes */
  udm_pos_t minpos;                  /* 4   bytes */
  udm_pos_t maxpos;                  /* 4   bytes */
  udm_secno_t secno;                 /* 1   byte  */
  udm_wordnum_t wordnum;             /* 1   byte  */
  udm_wordnum_t order;               /* 1   byte  */
} UDM_SECTION;                       /* 32 bytes (i386), 40 bytes (64bit) */


typedef struct udm_sectionlist_st
{
  size_t mcoords;
  size_t ncoords;
  UDM_COORD2 *Coord;
  size_t msections;
  size_t nsections;
  UDM_SECTION *Section;
} UDM_SECTIONLIST;


typedef struct udm_sectionlistlist_st
{
  size_t nitems;
  size_t mitems;
  UDM_SECTIONLIST *Item;
} UDM_SECTIONLISTLIST;



/** Main search structure */
typedef struct {
  urlid_t url_id;
  uint4   score;
} UDM_URL_SCORE;

typedef struct {
  size_t nitems;
  UDM_URL_SCORE	*Item;
} UDM_URLSCORELIST;


/* UserScore and UserSiteScore structure */

typedef struct udm_url_int4_st
{
  urlid_t url_id;
  int4  param;
} UDM_URL_INT4;

typedef struct udm_url_int4_list_st
{
  size_t nitems;
  UDM_URL_INT4 *Item;
} UDM_URL_INT4_LIST;



/* Structure to handle limits */
typedef struct udm_urlid_list_st
{
  char empty;
  char exclude;
  urlid_t *urls;
  size_t nurls;
} UDM_URLID_LIST;


typedef struct {
        urlid_t         url_id;
        uint4           score;
        uint4           per_site;
        urlid_t         site_id;
        time_t          last_mod_time;
        double          pop_rank;
	char            *url;
        char            *section;
} UDM_URLDATA;

#define UDM_COORD2DBNUM(score) (255 - (int) ((score) & 0xFF))

typedef struct {
	size_t		nitems;
	UDM_URLDATA	*Item;
} UDM_URLDATALIST;


/** Word list unit */
typedef struct
{
  char		*word;
  UDM_COORD     coord;
  unsigned char hash;
  unsigned char seclen_marker;
} UDM_WORD;

typedef struct
{
  size_t    wordpos[256]; /**< Word positions in sections               */
  size_t    mwords;       /**< Number of memory allocated for words     */
  size_t    nwords;       /**< Real number of words in list             */
  UDM_WORD *Word;         /**< Word list  itself                        */
} UDM_WORDLIST;


typedef struct
{
  size_t nitems;
  UDM_WORDLIST Item[256];
} UDM_WORDLISTLIST;


typedef struct
{
  const char *str;      /* 4 */
  udmcrc32_t crc;       /* 4 */
  UDM_COORD coord;      /* 4 */
  unsigned char length; /* 1 */
} UDM_CONSTWORD;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_CONSTWORD *Item;
  udm_pos_t wordpos[256];
} UDM_CONSTWORDLIST;

/***************************************************************/

/* Server/Realm follow types */
typedef enum
{
  UDM_WEBSPACE_PAGE= 0,
  UDM_WEBSPACE_PATH= 1,
  UDM_WEBSPACE_SITE= 2,
  UDM_WEBSPACE_WORLD= 3,
  UDM_WEBSPACE_URLLIST= 4,
  UDM_WEBSPACE_UNKNOWN= 127
} udm_webspace_t;
#define UDM_WEBSPACE_DEFAULT  UDM_WEBSPACE_PATH

typedef struct
{ 
  udm_bool_t index;   /**< Whether to index words         */
  udm_bool_t follow;  /**< Whether follow links           */
  udm_bool_t archive; /**< Whether to store cached copies */
} UDM_ROBOTSPARAM;


typedef struct
{
  int max_net_errors;
  int net_error_delay_time;
  int read_timeout;
  int doc_timeout;
  int period;                   /**< Reindex period           */
  int maxhops;                  /**< Max way in mouse clicks  */
  int doc_per_site;
  int crawl_delay;
  int dns_cache_timeout;
  int link_sources_to_follow;
  udm_link_destination_t collect_links_destination;
  udm_link_format_t      collect_links_format;
  udm_webspace_t webspace;      /**< World, Site, Path, Page  */
  udm_use_robots_t use_robots;  /**< Whether to use robots.txt and meta tags */
  UDM_ROBOTSPARAM robots;
  udm_bool_t use_clones;        /**< Whether to detect clones */
} UDM_SPIDERPARAM;

/*****************************************************************/

#define UDM_MATCH_FLAG_SKIP_OPTIMIZATION 1

typedef struct
{
  int one;
  int many;
  int eol;
} UDM_WILD_PARAM;


typedef struct {
	int		match_type;
	int		nomatch;
	int		case_sense;
	int		flags; /* e.g. skip normalizaton */
	char		*section;
	char		*pattern;
	size_t          pattern_length;
	regex_t		*reg;
	char		*arg;
	char		*arg1;
} UDM_MATCH;

typedef struct {
	size_t		nmatches;
	UDM_MATCH	*Match;
} UDM_MATCHLIST;

typedef struct {
	int beg;
	int end;
} UDM_MATCH_PART;



/*****************************************************************/
/** Parsed URL string */
typedef struct udm_url
{
  char *schema;
  char *specific;
  char *hostinfo;
  char *auth;
  char *hostname;
  char *path;
  char *filename;
  char *anchor;
  int  port;
  int  default_port;
} UDM_URL;


/** List of URLs */
typedef struct udm_url_list
{
  size_t nitems;
  size_t mitems;
  UDM_URL *Item;
} UDM_URLLIST;


/*****************************************************************/

/** Structure to store server parameters */
typedef struct
{
  UDM_MATCH    Match;
  urlid_t      site_id;        /**< server.rec_id            */
  char         command;        /**< 'S' - server,realm, 'F' - disallow,allow */
  int          ordre;          /**< order in list to find    */
  urlid_t      parent;         /**< parent rec_id for grouping by site */
  float        weight;         /**< server weight for popularity rank calculation */
  UDM_VARLIST  Vars;           /**< Default lang, charset,etc*/
  UDM_URLLIST  ProxyList;      /**< List of proxies */
  uint4        MaxHops;
  udm_webspace_t webspace;     /* Page, Path, Site, World, etc*/
  udm_method_t method;         /* Allow, Disallow, etc */
  udm_bool_t   enabled;
} UDM_SERVER;


typedef struct {
  size_t    nservers;
  size_t    mservers;
  int    have_subnets;
  UDM_SERVER  *Server;
} UDM_SERVERLIST;


/*******************************************************/
/* All links are stored in the cache of this structure */
/* before actual INSERT into database                  */

typedef struct
{
  char *url;
  urlid_t referrer;
  uint4	hops;
  urlid_t site_id;
  urlid_t server_id;
  urlid_t rec_id;
  size_t  max_doc_per_site;
  UDM_VARLIST HrefVars;
  udm_method_t method;
  udm_bool_t stored;
  udm_link_source_t link_source;
  udm_link_destination_t method_reason;
} UDM_HREF;

typedef enum udm_hreflist_flag_enum
{
  UDM_HREFLIST_FLAG_NONUNIQ= 1
} udm_hreflist_flag_t;

typedef struct
{
  size_t    mhrefs;
  size_t    nhrefs;
  size_t    shrefs;
  size_t    dhrefs;
  UDM_HREF  *Href;
  int       flags;
} UDM_HREFLIST;

/*******************************************************/

/** Resolve stuff */
typedef struct udm_host_addr_struct {
	char		*hostname;
	struct in_addr	addr;
	int		net_errors;
	time_t		last_used;
	time_t          expires;
}UDM_HOST_ADDR;

typedef struct {
	size_t		nhost_addr;
	size_t		mhost_addr;
	UDM_HOST_ADDR	*host_addr;
} UDM_HOSTLIST;

/** Used in FTP sessions */
typedef struct udm_conn_struct {
        int	status;
        int	connected;
        int	err;
        time_t	host_last_used;
        int	conn_fd;
#ifdef WIN32
        unsigned short port;
#else
        int	port;
#endif
        int	timeout;
        char	*hostname;
        char    *user;
        char    *pass;
        struct	sockaddr_in sin;
        int	buf_len;
        size_t	buf_len_total;
        int	len;
        char	*buf;
        int     net_errors;
        struct	udm_conn_struct *connp;
} UDM_CONN;

/***************************************************/

typedef struct
{
  char   *buf;           /**< Buffer to download document to    */
  char   *content;       /**< Pointer to content, after headers */
  size_t  size;          /**< Number of bytes loaded            */
  size_t  alloced_size;   /**< Maximum bytes to load into buf    */
} UDM_HTTPBUF;


typedef struct
{
  int freeme;           /**< Whether  memory was allocated for doc   */
  int stored;           /**< If it is already stored, forAddHref()   */
  udm_method_t method;  /**< How to download document: GET, HEAD etc */

  UDM_HTTPBUF     Buf;                  /**< Buffer       */

  UDM_HREFLIST    Hrefs;                /**< Link list    */

  UDM_VARLIST     RequestHeaders;       /**< Extra headers*/
  UDM_VARLIST     Sections;             /**< User sections*/

  UDM_TEXTLIST    TextList;             /**< Text list    */
  UDM_URL         CurURL;               /**< Parsed URL   */
  UDM_CHARSET     *lcs;                 /**< LocalCharser */
  UDM_SPIDERPARAM Spider;               /**< Spider prms  */
  UDM_CONN        connp;                /**< For FTP      */
  UDM_CONN        connp2;               /**< For FTP      */
} UDM_DOCUMENT;

/********************************************************/

/** External Parsers */
typedef struct udm_parser_struct{
        char		*from_mime;
	char		*to_mime;
	char		*cmd;
	char		*src;
} UDM_PARSER;

typedef struct {
	size_t		nparsers;
	UDM_PARSER	*Parser;
} UDM_PARSERLIST;


/******* Ispell BEGIN ********/

#define UDM_SPELL_NOPREFIX 1

typedef struct udm_spell_st
{
  char *word;
  char *flags;
} UDM_SPELL;

#define UDM_SPELL_LANGLEN 32
#define UDM_SPELL_CSETLEN 32
#define UDM_SPELL_FILELEN 128
#define UDM_SPELL_FMT_TEXT 0
#define UDM_SPELL_FMT_HASH 1

typedef struct udm_dict_st
{
  char  lang[UDM_SPELL_LANGLEN];
  char  cset[UDM_SPELL_CSETLEN];
  char  fname[UDM_SPELL_FILELEN];
  int   fmt;
  int   fd;
  size_t wordlen;
  UDM_CHARSET *cs;
  char   *fbody;
  size_t nitems;
  size_t mitems;
  UDM_SPELL *Item;
} UDM_SPELLLIST;


typedef struct udm_spelllistlist_st
{
  size_t nitems;
  size_t mitems;
  /*size_t nspell;*/        /* Backward capability with php-4.3.x */
  UDM_SPELLLIST *Item;
} UDM_SPELLLISTLIST;


typedef struct udm_aff_st
{
  char flag;
  char type;
  regex_t regex;
  char *find;
  char *repl;
  char *mask;
  size_t findlen;
  size_t replen;
} UDM_AFFIX;


typedef struct udm_afflist_st
{
  size_t mitems;
  size_t nitems;
  char  lang[UDM_SPELL_LANGLEN];
  char  cset[UDM_SPELL_CSETLEN];
  char  fname[UDM_SPELL_FILELEN];
  UDM_CHARSET *cs;
  UDM_AFFIX *Item;
} UDM_AFFIXLIST;


typedef struct udm_afflistlist_st
{
  size_t mitems;
  size_t nitems;
  UDM_AFFIXLIST *Item;
} UDM_AFFIXLISTLIST;


/******* Ispell END **********/


typedef struct
{
  udm_method_t  method; /**< 'allow' or 'disallow' */
  char         *path;
} UDM_ROBOT_RULE;


typedef struct{
	char		*hostinfo;
	size_t		nrules;
	UDM_ROBOT_RULE	*Rule;
} UDM_ROBOT;

typedef struct{
	size_t		nrobots;
	UDM_ROBOT	*Robot;
} UDM_ROBOTS;


typedef struct {
  size_t order;
  size_t order_extra_width; /* For multi-word synonyms, see below */
  size_t count;
  size_t doccount; /* Number of documents this word appears in */
  int    origin;
  int    weight; /* origin-dependent weight   */
  int    user_weight; /* User-supplied weight */
  int    match;  /* BEGIN,END,SUBSTR,NUM_LT,NUM_GT,FULL*/
  size_t secno;  /* Which section to search in */
  size_t phrpos; /* 0 means "not in phrase"    */
  size_t phrlen; /* phase length               */
  size_t phrwidth; /* How many additional parts in a multi-word */
} UDM_WIDEWORD_PARAM;


typedef struct {
  UDM_STR Word;
  UDM_WIDEWORD_PARAM Param;
} UDM_WIDEWORD;


/*
  order_extra_width - use in case of many-to-one and many-to-many synonyms.
  It represents the number of query words this synonym covers.
  For example, if we have synonym:
  "aaaa bbbb" -> "cccc"
  then origin_extra_width for the words "cccc" will be 2, because
  it is a replacement for two words.
  0 means "not a multi-word replacement".
*/

typedef struct {
	int		wm;              /* Search mode: wrd, sub, beg, end */
	int             strip_noaccents; /* If accent insensitive comparison*/
	size_t		nuniq;
	size_t		nwords;
	UDM_WIDEWORD	*Word;
} UDM_WIDEWORDLIST;


typedef struct {
	char	*p;
	char	*s;
	int     origin; /* SYNONYM or SYNONYM_FINAL */
} UDM_SYNONYM;

#define UDM_SYNONYM_LANGLEN 32
#define UDM_SYNONYM_CSETLEN 32
#define UDM_SYNONYM_FILELEN 128
#define UDM_SYNONYM_FMT_TEXT 0
#define UDM_SYNONYM_FMT_HASH 1

typedef struct {
	size_t		nsynonyms;
	size_t		msynonyms;
	UDM_SYNONYM	*Synonym;
        char  lang[UDM_SYNONYM_LANGLEN];
        char  cset[UDM_SYNONYM_CSETLEN];
        char  fname[UDM_SYNONYM_FILELEN];
        size_t max_phrase_length; /* Many-to-many, many-to-one, one-to-many */
} UDM_SYNONYMLIST;


typedef struct
{
  size_t nitems;
  UDM_SYNONYMLIST *Item;
} UDM_SYNONYMLISTLIST;


typedef struct udm_chinaword_struct {
  int *word;
  int  freq;
} UDM_CHINAWORD;

typedef struct {
  size_t        nwords, mwords;
  size_t        total;
  UDM_CHINAWORD *ChiWord;
  size_t        *hash;
} UDM_CHINALIST;


typedef struct udm_category_struct {
	int		rec_id;
	char		path[128];
	char		link[128];
	char		name[128];
} UDM_CATITEM;

typedef struct {
	char		addr[128];
	size_t		ncategories;
	UDM_CATITEM	*Category;
} UDM_CATEGORY;

/* Boolean search constants and types */
#define UDM_MAXSTACK	 128
#define UDM_STACK_LEFT	 0
#define UDM_STACK_RIGHT	 1
#define UDM_STACK_BOT	 2
#define UDM_STACK_OR	 3
#define UDM_STACK_AND	 4
#define UDM_STACK_NOT	 5
#define UDM_STACK_PHRASE 6
#define UDM_STACK_WORD	 200
#define UDM_STACK_STOP   201

typedef struct {
	size_t		ncstack, mcstack;
	int		*cstack;
	size_t		nastack, mastack;
	unsigned long	*astack;
} UDM_BOOLSTACK;

typedef struct {
	int		cmd;
	unsigned long	arg;
} UDM_STACK_ITEM;

typedef struct {
        size_t nitems;
        size_t mitems;
        size_t ncmds;
        UDM_STACK_ITEM *items;
} UDM_STACKITEMLIST;

typedef struct {
	size_t			work_time;
	size_t			first;
	size_t			last;
	size_t			total_found;
	size_t			num_rows;
	size_t			cur_row;
	size_t			memused;
	int			freeme;
	UDM_DOCUMENT		*Doc;
	
	UDM_WIDEWORDLIST	WWList;
	UDM_URLDATALIST		URLData;
	UDM_STACKITEMLIST	ItemList;
	
} UDM_RESULT;


#include "udm_db_int.h"

typedef struct {
  size_t      nitems;
  size_t      currdbnum;
  UDM_DB     *db;
  char errstr[128];
} UDM_DBLIST;


#define UDM_LOG_FLAG_SKIP_PID 0x00000001

typedef struct
{
  int is_log_open; /* Flag indicating if openlog() has been called      */
  FILE *logFD;     /* File descriptor, when logging to stderr or file   */
  int facility;    /* Which facility to use, or negative number if none */
  int flags;
} UDM_LOG;


/** Forward declaration of UDM_AGENT */
struct udm_indexer_struct;

/** Config file */
typedef struct udm_config_struct {
	int		freeme;
	char		errstr[2048];
	UDM_CHARSET	*bcs;
	UDM_CHARSET	*lcs;
	
	int		url_number;	/**< For indexer -nXXX          */
	
	UDM_SERVERLIST	Servers;	/**< List of servers and realms */
        UDM_SERVER      *Cfg_Srv;
	
	UDM_MATCHLIST	Aliases;	/**< Straight aliases           */
	UDM_MATCHLIST	ReverseAliases;	/**< Reverse aliases            */
	UDM_MATCHLIST	MimeTypes;	/**< For AddType commands       */
	UDM_MATCHLIST	Filters;	/**< Allow, Disallow,etc        */
	UDM_MATCHLIST	SectionFilters; /**< IndexIf, NoIndexIf         */
        UDM_MATCHLIST   SectionHdrMatch;/**< User sections after headers*/
        UDM_MATCHLIST   SectionGsrMatch;/**< User sections after quesser*/
        UDM_MATCHLIST   SectionMatch;   /**< User sections after parser */
        UDM_MATCHLIST	Encodings;	/**< For AddType commands       */
	
	UDM_HREFLIST	Hrefs;		/**< Links cache                */
	UDM_RESULT	Targets;	/**< Targets cache              */
	
	UDM_VARLIST	Sections;	/**< document section to parse  */
	UDM_VARLIST	Vars;		/**< Config parameters          */
	UDM_VARLIST	Cookies;	/**< Cookie list                */
	UDM_VARLIST	XMLEnterHooks;
	UDM_VARLIST	XMLLeaveHooks;
	UDM_VARLIST	XMLDataHooks;
	
	UDM_LANGMAPLIST	LangMaps;	/**< For lang+charset quesser   */
	UDM_ROBOTS	Robots;		/**< robots.txt information     */
	UDM_SYNONYMLISTLIST Synonym;    /**< Synonims list              */
	UDM_STOPLISTLIST StopWord;      /**< Stopwords list             */
	UDM_PARSERLIST	Parsers;	/**< External  parsers          */
	UDM_DBLIST	dbl;		/**< Searchd addresses	      */
	UDM_HOSTLIST	Hosts;		/**< Resolve cache              */
	UDM_SPELLLISTLIST Spells;	/**< For ispell dictionaries    */
	UDM_AFFIXLISTLIST Affixes;	/**< For ispell affixes         */
	UDM_WORDPARAM	WordParam;	/**< Word limits                */
        UDM_CHINALIST   Chi;            /**< Chinese words list         */
        UDM_CHINALIST   Thai;           /**< Thai words list            */
	
	UDM_LOG Log;
	
	int		CVS_ignore;	/**< Skip CVS directgories - for tests */
	
	/* Various virtual functions */
	void (*ThreadInfo)(struct udm_indexer_struct *,const char * state,const char * str);
	void (*LockProc)(struct udm_indexer_struct *,int command,int type,const char *fname,int lineno);
	void (*RefInfo)(int code,const char *url, const char *ref);
	int  (*DumpDoc)(struct udm_indexer_struct *, UDM_DOCUMENT *Doc);
	int (*ThreadCreate)(void **thd, void *(*start_routine)(void *), void *arg);
	int (*ThreadJoin)(void *thd);
#ifdef MECAB
        mecab_t         *mecab;
#endif
        UDM_UNIDATA *unidata;

} UDM_ENV;


#define UDM_AGENT_STATE_MUTEXES 5

typedef struct udm_agent_state_t
{
  const char *task;
  const char *param;
  const char *extra;
  time_t start_time;
  size_t mutex_owned[UDM_AGENT_STATE_MUTEXES];
  size_t nmutexes;
} UDM_AGENT_STATE;

/** Indexer */
typedef struct udm_indexer_struct{
	int		freeme;		/**< whenever it was allocated    */
	int		handle;		/**< Handler for threaded version */
	time_t		start_time;	/**< Time of allocation, for stat */
	udm_uint8   nbytes;     /**< Number of bytes downloaded   */
	size_t		ndocs;		/**< Number of documents          */
        size_t          nsleepsecs;     /**> Number of sleep seconds      */
	int		flags;		/**< Callback function to request action*/
        int             action;
	int		doccount;	/**< for UdmGetDocCount()         */
	UDM_ENV		*Conf;		/**< Configuration                */
	UDM_LANGMAP	*LangMap;	/**< LangMap for current document */
	UDM_RESULT	Indexed;	/**< Indexed cache              */

	UDM_AGENT_STATE State;
	
        char    *UdmFindURLCache[UDM_FINDURL_CACHE_SIZE];
        urlid_t UdmFindURLCacheId[UDM_FINDURL_CACHE_SIZE];
        size_t  pURLCache;
        char    *ServerIdCache[UDM_SERVERID_CACHE_SIZE];
        char    ServerIdCacheCommand[UDM_SERVERID_CACHE_SIZE];
        urlid_t ServerIdCacheId[UDM_SERVERID_CACHE_SIZE];
        size_t  pServerIdCache;

#ifdef USE_TRACE
  FILE *TR;
#endif
	
} UDM_AGENT;



typedef struct {
	char	*url;
	int	status;
} UDM_URLSTATE;

typedef int (*udm_qsort_cmp)(const void*, const void*);

typedef struct {
	uint4	hi,lo;
	off_t	pos;
	size_t	len;
} UDM_UINT8_POS_LEN;

typedef struct {
	uint4	val;
	off_t	pos;
	size_t	len;
} UDM_UINT4_POS_LEN;

typedef struct {
	uint4	val;
	urlid_t	url_id;
} UDM_UINT4URLID;

typedef struct {
	size_t		nitems;
	UDM_UINT4URLID	*Item;
} UDM_UINT4URLIDLIST;

typedef struct {
	uint4	hi,lo;
	urlid_t	url_id;
} UDM_UINT8URLID;

typedef struct {
	size_t		nitems;
	UDM_UINT8URLID	*Item;
} UDM_UINT8URLIDLIST;


#define UDM_LOGD_CMD_WORD  0
#define UDM_LOGD_CMD_DATA  1
#define UDM_LOGD_CMD_CHECK 2


typedef struct udm_cfg_st {
        UDM_AGENT       *Indexer;
	UDM_SERVER	*Srv;
	int		flags;
	int		level;
	int		ordre;
} UDM_CFG;

#ifndef udm_max
#define udm_max(a,b) (((a) > (b)) ? (a) : (b))
#endif
#ifndef udm_min
#define udm_min(a,b) (((a) < (b)) ? (a) : (b))
#endif

#ifdef DMALLOC
#include <dmalloc.h>
#endif

#define UDM_DT_BACK    1
#define UDM_DT_ER      2
#define UDM_DT_RANGE   3
#define UDM_DT_UNKNOWN 4

typedef enum udm_opttype_en
{
  UDM_OPT_BOOL,
  UDM_OPT_INT,
  UDM_OPT_STR,
  UDM_OPT_TITLE
} udm_opttype_t;


typedef struct udm_cmdline_opt_st
{
  int id;
  const char *name;
  int type;
  void *value;
  const char *comment;
} UDM_CMDLINE_OPT;


enum udm_indcmd
{
  UDM_IND_AMBIGUOUS=     0,
  UDM_IND_UNKNOWN=       1,
  UDM_IND_INDEX=         300,
  UDM_IND_CRAWL=         301,
  UDM_IND_STAT=          'S',
  UDM_IND_CREATE=        303,
  UDM_IND_DROP=          304,
  UDM_IND_DELETE=        'C',
  UDM_IND_REFERERS=      'I',
  UDM_IND_SQLMON=        'Q',
  UDM_IND_CHECKCONF=     308,
  UDM_IND_CONVERT=       309,
  UDM_IND_MULTI2BLOB=    310,
  UDM_IND_EXPORT=        311,
  UDM_IND_WRDSTAT=       312,
  UDM_IND_REWRITEURL=    313,
  UDM_IND_HASHSPELL=     314,
  UDM_IND_DUMPSPELL=     315,
  UDM_IND_REWRITELIMITS= 316,
  UDM_IND_DUMPCONF=      317,
  UDM_IND_DUMPDATA=      318,
  UDM_IND_RESTOREDATA=   319,
  UDM_IND_EXECSQL=       320,
  UDM_IND_SET=           321, /* indexer --set=a=b -> a=b    */
  UDM_IND_SET0=          322  /* indexer --fl=xxx  -> fl=xxx */
};

#endif
