/* Copyright (C) 2000-2013 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#ifndef _UDM_PARSE_HTML_H
#define _UDM_PARSE_HTML_H

/* HTML tag types */
enum udm_html_tag
{
  UDM_HTML_TAG_UNKNOWN= 0,
  UDM_HTML_TAG_A,
  UDM_HTML_TAG_TITLE,
  UDM_HTML_TAG_HTML,
  UDM_HTML_TAG_BODY,
  UDM_HTML_TAG_META,
  UDM_HTML_TAG_BASE,
  UDM_HTML_TAG_LINK,
  UDM_HTML_TAG_AREA,
  UDM_HTML_TAG_IMG,
  UDM_HTML_TAG_FRAME,
  UDM_HTML_TAG_IFRAME,
  UDM_HTML_TAG_SCRIPT,
  UDM_HTML_TAG_STYLE,
  UDM_HTML_TAG_NOINDEX,
  UDM_HTML_TAG_FONT,
  UDM_HTML_TAG_P
};


#define UDM_MAXTAGVAL	64

typedef struct udm_html_tag_st
{
  size_t ntoks;
  UDM_CONST_ATTR toks[UDM_MAXTAGVAL + 1];
} UDM_HTML_TAG;


extern void UdmHTMLTagInit(UDM_HTML_TAG *tag);
extern size_t UdmHTMLTagParse(UDM_HTML_TAG *tag, const char *src, size_t length);
extern UDM_CONST_STR *UdmHTMLTagFindAttrByName(UDM_HTML_TAG *tag, const char *name, size_t namelen);

/************************************************/

/* HTML parser states */
enum udm_html_token
{
  UDM_HTML_TOKEN_UNKNOWN= 0,
  UDM_HTML_TOKEN_TAG=     1,
  UDM_HTML_TOKEN_TEXT=    2,
  UDM_HTML_TOKEN_COMMENT= 3
};


typedef struct udm_html_parser_state_st
{
  char script;   /* if inside <script>..</script> */
  char style;    /* if inside <style>..</style>   */
  char title;    /* if inside <title>..</title>   */
  char body;     /* if inside <body>..</body>     */
  char comment;  /* if inside <!--UdmComment--> <!--/UdmComment--> */
} UDM_HTML_PARSER_STATE;


typedef struct udm_html_parser_st
{
  UDM_HTML_PARSER_STATE state;
  UDM_CONST_STR lasttok;
  UDM_HTML_TAG tag;
  void *user_data;
  int (*tag_handler)(struct udm_html_parser_st *st);
  int (*comment_handler)(struct udm_html_parser_st *st);
  int (*text_handler)(struct udm_html_parser_st *st);
} UDM_HTML_PARSER;



extern int UdmHTMLParserInit(UDM_HTML_PARSER *parser);
extern void UdmHTMLParserSetUserData(UDM_HTML_PARSER *parser, void *data);
extern int UdmHTMLParserExec(UDM_HTML_PARSER *parser, const char *src, size_t strlen);

extern void UdmHTMLParserSetTextHandler(UDM_HTML_PARSER *p, int (*action)(UDM_HTML_PARSER*));
extern void UdmHTMLParserSetTagHandler(UDM_HTML_PARSER *p, int (*action)(UDM_HTML_PARSER*));
extern void UdmHTMLParserSetCommentHandler(UDM_HTML_PARSER *p, int (*action)(UDM_HTML_PARSER*));

/**************************************************/

extern int    UdmHTMLParse(UDM_AGENT*, UDM_DOCUMENT*);
extern int UdmParseURLText(UDM_AGENT*, UDM_DOCUMENT*);
extern int    UdmParseText(UDM_AGENT*, UDM_DOCUMENT*);
extern int UdmParseHeaders(UDM_AGENT*, UDM_DOCUMENT*);
extern int UdmMessageRFC822Parse(UDM_AGENT *, UDM_DOCUMENT *);
extern size_t UdmMessageRFC822ExcerptSource(UDM_AGENT *Agent,
                                            UDM_RESULT *Res,
                                            UDM_DOCUMENT *Doc,
                                            const UDM_CONST_STR *content,
                                            UDM_DSTR *dst);
extern int UdmMessageRFC822CachedCopy(UDM_AGENT *Agent,
                                      UDM_RESULT *Res,
                                      UDM_DOCUMENT *Doc,
                                      UDM_DSTR *dstr);
extern int UdmRTFParse(UDM_AGENT *, UDM_DOCUMENT *);
extern int UdmRTFCachedCopy(UDM_AGENT *Agent,
                             UDM_RESULT *Res,
                             UDM_DOCUMENT *Doc,
                             UDM_DSTR *dstr);
extern size_t UdmRTFExcerptSource(UDM_AGENT *Agent,
                                  UDM_RESULT *Res,
                                  UDM_DOCUMENT *Doc,
                                  const UDM_CONST_STR *content,
                                  UDM_DSTR *dst);
extern int UdmDOCXParse(UDM_AGENT *, UDM_DOCUMENT *);
extern int UdmDOCXCachedCopy(UDM_AGENT *Agent,
                             UDM_RESULT *Res,
                             UDM_DOCUMENT *Doc,
                             UDM_DSTR *dstr);
extern size_t UdmDOCXExcerptSource(UDM_AGENT *Agent,
                                   UDM_RESULT *Res,
                                   UDM_DOCUMENT *Doc,
                                   const UDM_CONST_STR *content,
                                   UDM_DSTR *dst);

extern int UdmHTDBParse(UDM_AGENT *, UDM_DOCUMENT *);
extern size_t UdmHTDBExcerptSource(UDM_AGENT *Agent,
                                   UDM_RESULT *Res,
                                   UDM_DOCUMENT *Doc,
                                   const UDM_CONST_STR *content,
                                   UDM_DSTR *dst);
extern size_t UdmConvRFC1522(UDM_CONV *conv, char *dst, size_t dstlen,
                             const char *src, size_t srclen, int flags);
extern int UdmPrepareWords(UDM_AGENT*, UDM_DOCUMENT*, UDM_WORDLIST *WL);
extern int UdmPrepareSections(UDM_AGENT *, UDM_DOCUMENT*);
extern int UdmPrepareRawSections(UDM_AGENT *, UDM_DOCUMENT *);

extern int UdmDocInternalParserExec(UDM_AGENT *, UDM_DOCUMENT *, udm_content_type_t);
#endif
