/* Copyright (C) 2000-2013 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#ifndef _UDM_SEARCH_TOOL_H
#define _UDM_SEARCH_TOOL_H

#define UDM_FAST_PRESORT_DOCS	300
#define UDM_DEFAULT_USER_WORD_WEIGHT 256


typedef struct
{
  UDM_CONV src_uni;
  UDM_CONV uni_dst;
  UDM_CONV uni_wcs;
  int hlstop;
  int segmenter;
} UDM_HIGHLIGHT_CONV;


/* Functions form urldata.c */
extern void UdmURLDataSortBySite(UDM_URLDATALIST *L);
extern void UdmURLDataSortByPattern(UDM_URLDATALIST *D, const char *pattern);
extern UDM_URLDATA *UdmURLDataListSearch(UDM_URLDATALIST *List, urlid_t id);
extern void UdmURLDataListFreeItems(UDM_URLDATALIST *List, size_t first, size_t last);
extern void UdmURLDataListFree(UDM_URLDATALIST *List);
extern int UdmURLDataListClearParams(UDM_URLDATALIST *List, size_t num_best_rows);
extern size_t UdmURLDataCompact(UDM_URLDATA *dst, UDM_URLDATA *src, size_t n);

/* Functions from groupby.c */
extern void UdmURLDataGroupBySite(UDM_URLDATALIST *L);
extern int UdmURLDataListGroupBySiteUsingSort(UDM_AGENT *A,
                                              UDM_URLDATALIST *R,
                                              UDM_DB *db);
extern void UdmURLDataApplySiteRank(UDM_AGENT *A, UDM_URLDATALIST *DataList,
                                    int is_aggregation_point);

int UdmURLDataListPackSite(UDM_URLDATALIST *List, UDM_DSTR *dstr);
int UdmURLDataListUnpackSite(UDM_AGENT *A, UDM_URLDATALIST *List,
                             const UDM_CONST_STR *str_arg);

/* Functions from distance.c */
typedef struct udm_distance_stat_st
{
  uint4 sum;
  uint4 num;
  uint4 phrase2_count;
  uint4 phrase3_count;
} UDM_WORD_DISTANCE_STAT;

void UdmWordDistanceStatInit(UDM_WORD_DISTANCE_STAT *d);

extern void CalcAverageWordDistance(size_t wf2_secno,
                                    UDM_COORD2 *phr, size_t num, size_t nuniq,
                                    UDM_WORD_DISTANCE_STAT *dist);
                                                 
/* Functions from score.c */
extern int UdmUserScoreListApplyToURLScoreList(UDM_AGENT *A,
                                               UDM_URLSCORELIST *URLScoreList,
                                               UDM_URL_INT4_LIST *UserScoreList,
                                               int UserScoreFactor);
extern int UdmUserScoreListApplyToURLDataList(UDM_AGENT *A,
                                              UDM_URLDATALIST *URLDataList,
                                              UDM_URL_INT4_LIST *UserScoreList,
                                              int UserScoreFactor);

extern int
UdmURLDataListApplyRelevancyFactors(UDM_AGENT *Agent,
                                    UDM_URLDATALIST *DataList,
                                    int RelevancyFactor,
                                    int DateFactor);

extern void UdmURLScoreListSortByScore(UDM_URLSCORELIST *ScoreList);
extern void UdmURLScoreListSortByScoreThenURLTop(UDM_URLSCORELIST *ScoreList,
                                                 size_t topcount);
extern void UdmURLScoreListSortByScoreThenURL(UDM_URLSCORELIST *ScoreList);

extern void UdmGroupByURL2(UDM_AGENT *Agent,
                           UDM_DB *db,
                           UDM_RESULT *Res,
                           UDM_SECTIONLIST *CoordList,
                           UDM_URLSCORELIST *ScoreList);

extern int  UdmPrepare(UDM_AGENT *query,UDM_RESULT *res);
extern __C_LINK int __UDMCALL UdmParseQueryString(UDM_AGENT * Agent,UDM_VARLIST * vars, const char * query_string);

/* Functions from wordinfo.c */
int UdmResWordInfo(UDM_ENV *Env, UDM_RESULT *Res);


/* Functions from fuzzy.c */
int UdmAllForms(UDM_AGENT *Indexer,
                UDM_WIDEWORDLIST *result,
                UDM_WIDEWORD *uword);

int UdmComplexSynonyms(UDM_AGENT *Indexer, UDM_WIDEWORDLIST *result);

/* Functions from highlight.c */
extern void UdmExcerptConvInit(UDM_HIGHLIGHT_CONV *cnv,
                               UDM_CHARSET *wcs,
                               UDM_CHARSET *src,
                               UDM_CHARSET *dst,
                               int hlstop, int segmenter);
extern void UdmExcerptConvInitFromEnv(UDM_HIGHLIGHT_CONV *cnv,
                                      UDM_CHARSET *wcs,
                                      UDM_CHARSET *src,
                                      UDM_CHARSET *dst,
                                      UDM_ENV *Env);
extern size_t
UdmHlConvertExtWithConv(UDM_AGENT *A, UDM_DSTR *dstr,
                        UDM_WIDEWORDLIST *List,
                        const char *src, size_t srclen,
                        UDM_HIGHLIGHT_CONV *ec);
extern int
UdmVarListHlConvert(UDM_AGENT *A,
                    UDM_WIDEWORDLIST *WWList,
                    UDM_VARLIST *Vars,
                    UDM_HIGHLIGHT_CONV *ec);

extern int  UdmConvert(UDM_AGENT *A, UDM_RESULT *Res,UDM_CHARSET *lcs,UDM_CHARSET *bcs);
extern char* UdmRemoveHiLightDup(const char *s);

/* Functions from segment.c */
int *UdmUniSegment(UDM_AGENT *Indexer, int *s, const char *lang, const char *seg);
int *UdmUniSegmentByType(UDM_AGENT *Indexer, int *s, int type, int ch);
int UdmUniSegmenterFind(UDM_ENV *Env, const char *lang, const char *seg);
int UdmTextListSegment(UDM_AGENT *Indexer, UDM_TEXTLIST *tlist,
                       UDM_CHARSET *cs, int type);

/* Functions from section.c */
void UdmSectionListPrint(UDM_SECTIONLIST *SectionList);
int UdmSectionListAlloc(UDM_SECTIONLIST *List, size_t ncoords, size_t nsections);
void UdmSectionListFree(UDM_SECTIONLIST *List);
int UdmSectionListListAdd(UDM_SECTIONLISTLIST *List, UDM_SECTIONLIST *Item);
void UdmSectionListListInit(UDM_SECTIONLISTLIST *List);
void UdmSectionListListFree(UDM_SECTIONLISTLIST *List);
int UdmSectionListListMergeSorted(UDM_SECTIONLISTLIST *SrcList,
                                  UDM_SECTIONLIST *Dst, int opt);

/* Functions from urlidlist.c */
int UdmCmpURLID(urlid_t *s1, urlid_t *s2); /* for qsort */
int UdmURLIdListJoin(UDM_URLID_LIST *urls, UDM_URLID_LIST *fl_urls);
int UdmURLIdListUnion(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
int UdmURLIdListCopy(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
int UdmURLIdListMerge(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
int UdmURLIdListSort(UDM_URLID_LIST *a);

/* Functions from coords.c */
void
UdmURLCRDListSortByURLThenSecnoThenPos(UDM_URLCRDLIST *L);


/* Functions from sql.c */
int
UdmUserSiteScoreListLoadAndApplyToURLDataList(UDM_AGENT *Agent,
                                              UDM_URLDATALIST *List,
                                              UDM_DB *db);
#endif
